#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

int main (int argc, char* argv[]) {
    if (argc != 2) {
        fprintf(stderr, "Usage: %s cmd arg\n", argv[0]);
        exit(2);
    }

    {
        char *newargv[] = { "/bin/sh", "-c", argv[1], NULL };

        execve("/bin/sh", newargv, NULL);
    }

    perror("execve");
    exit(1);

    /* Execution should never reach here */
    return 1;
}
